from abc import ABC, abstractmethod

from cpl_core.configuration import ConfigurationABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.logging import LoggingSettings, Logger, LoggingLevelEnum, LoggerABC
from cpl_core.time import TimeFormatSettings

from bot_core.configuration.file_logging_settings import FileLoggingSettings


class CustomFileLoggerABC(Logger, ABC):

    @abstractmethod
    def __init__(self, key: str, config: ConfigurationABC, time_format: TimeFormatSettings, env: ApplicationEnvironmentABC):
        self._key = key
        self._settings: LoggingSettings = config.get_configuration(f'{FileLoggingSettings.__name__}_{key}')
        Logger.__init__(self, self._settings, time_format, env)
        self._begin_log()

    @property
    def settings(self) -> LoggingSettings:
        return self._settings

    def _begin_log(self):
        console_level = self._console.value
        self._console = LoggingLevelEnum.OFF
        self.info(__name__, f'Starting...')
        self._console = LoggingLevelEnum(console_level)

    def _get_string(self, name_list_as_str: str, level: LoggingLevelEnum, message: str) -> str:
        names = name_list_as_str.split(' ')
        log_level = level.name
        string = f'<{self._get_datetime_now()}> [ {log_level} ]'
        for name in names:
            string += f' [ {name} ]'
        string += f': {message}'
        return string

    def header(self, string: str):
        super().header(string)

    def trace(self, name: str, message: str):
        name = f'{name} {self._key}'
        super().trace(name, message)

    def debug(self, name: str, message: str):
        name = f'{name} {self._key}'
        super().debug(name, message)

    def info(self, name: str, message: str):
        name = f'{name} {self._key}'
        super().info(name, message)

    def warn(self, name: str, message: str):
        name = f'{name} {self._key}'
        super().warn(name, message)

    def error(self, name: str, message: str, ex: Exception = None):
        name = f'{name} {self._key}'
        super().error(name, message, ex)

    def fatal(self, name: str, message: str, ex: Exception = None):
        name = f'{name} {self._key}'
        super().fatal(name, message, ex)
