from typing import Callable

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC

from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from cpl_translation import TranslatePipe
from discord import app_commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC


class ClientUtilsService(ClientUtilsServiceABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: LoggerABC,
            bot: DiscordBotServiceABC,
            servers: ServerRepositoryABC,
            clients: ClientRepositoryABC,
            message_service: MessageServiceABC,
            db: DatabaseContextABC,
            t: TranslatePipe,
            feature_flags: FeatureFlagsSettings
    ):
        ClientUtilsServiceABC.__init__(self)
        self._config = config
        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._clients = clients
        self._message_service = message_service
        self._db = db
        self._t = t
        self._feature_flags = feature_flags

    def received_command(self, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        client.received_command_count += 1
        self._clients.update_client(client)
        self._db.save_changes()

    def moved_user(self, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        client.moved_users_count += 1
        self._clients.update_client(client)
        self._db.save_changes()

    def moved_users(self, guild_id: int, count: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        client.moved_users_count += count
        self._clients.update_client(client)
        self._db.save_changes()

    def get_client(self, dc_ic: int, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.server_id)
        return client

    async def check_if_bot_is_ready_yet(self) -> bool:
        if self._config.get_configuration('IS_READY') == 'true':
            return True

        self._logger.debug(__name__, f'Bot is not ready yet {self._t.transform("common.errors.bot_not_ready_yet")}')
        return False

    async def check_if_bot_is_ready_yet_and_respond(self, ctx: Context) -> bool:
        result = await self.check_if_bot_is_ready_yet()
        if not result:
            await self._message_service.send_ctx_msg(ctx, self._t.transform('common.errors.bot_not_ready_yet'), without_tracking=True)

        return result

    async def presence_game(self, t_key: str):
        if not self._feature_flags.get_flag(FeatureFlagsEnum.presence):
            return

        import bot
        name = self._t.transform(t_key).format(bot.__version__)
        await self._bot.change_presence(activity=discord.Game(name=name))
        self._logger.info(__name__, f'Set presence {name}')

    def get_auto_complete_list(self, _l: List, current: str, select: Callable = None) -> List:
        if current != '':
            if select is None:
                select = lambda x: x

            sl = _l.select(select)
            sl = sl.where(lambda x: current in x)
            if current in sl:
                sl = sl.skip(sl.index_of(current))

            _l = _l.where(lambda x: x.name in sl)

        return _l.take(25)
